@@ -4,12 +4,9 @@
 #####
 ############################
 ############################
-
 undir.path <- function(G, y, x) {
-
   ## G is an adjacency matrix
   ## y and x are two nodes
-
   geita <- geit <- which( G[x, ] == 1 )
   geita2 <- geit2 <- which( G[y, ] == 1 )
   
@@ -23,13 +20,13 @@ undir.path <- function(G, y, x) {
 
     for (i in geit)  geita <- c( geita, which(G[i, ] == 1) )
     while ( length(intersect(y, geita) ) == 0 )  {
-      for (i in geita)  geita <- c( geita, which(G[i, ] == 1) )
+      for (i in geita)   geita <- c( geita, which(G[i, ] == 1) )
       geita <- unique(geita)
     } 
 
     for (i in geit2)  geita2 <- c( geita2, which(G[i, ] == 1) )
     while ( length(intersect(x, geita2) ) == 0 )  {
-      for (i in geita2)  geita2 <- c( geita2, which(G[i, ] == 1) )
+      for (i in geita2)   geita2 <- c( geita2, which(G[i, ] == 1) )
       geita2 <- unique(geita2) 
     }
 
@@ -43,15 +40,10 @@ undir.path <- function(G, y, x) {
       xin <- length( intersect(a, x) )
       yin <- length( intersect(a, y) )
       gein <- length( intersect(a, geit) )
-      if ( xin + yin +  gein <= 1 )  {
-        ina[ which(geit == i) ] <- 0
-      }
+      if ( xin + yin +  gein <= 1 )   ina[ which(geit == i) ] <- 0
     }
-    
     geit <- c(y, geit[ ina ], x)
     
   }
-  
   geit
-  
 }
\ No newline at end of file
